using System;
using System.Drawing;

/*************************************************************************************
 *
 *  P4Lib
 *  Author : Bidou (http://www.csharpfr.com/auteurdetail.aspx?ID=13319)
 *  Blog   : http://blogs.developpeur.org/bidou/
 *  Date   : February 2007
 * 
 *************************************************************************************/

namespace P4Lib
{
    /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /// <summary>
    /// The different difficulties.
    /// </summary>
    /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public enum Difficulty
    {
        Hard,
        Average,
        Easy
    }

    /// ---------------------------------------------------------------------
    /// <summary>
    /// The Artificial Intelligency for the Tic-Tac-Toe game.
    /// </summary>
    /// ---------------------------------------------------------------------
    public class AI
    {
        private Difficulty _diff = Difficulty.Hard;

        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Create the AI.
        /// </summary>
        /// <param name="diff"> The difficulty. </param>
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        public AI(Difficulty diff)
        {
            this._diff = diff;
        }

        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Calculate the best column...
        /// </summary>
        /// <param name="game"> The current game. </param>
        /// <returns> The best column choosen by the AI. </returns>
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        public int GetBestColumn(Game game)
        {
            int bestCol = 0;
            switch (this._diff)
            {
                case Difficulty.Easy: bestCol = AlphaBetaAlgo.AlphaBeta(game, 1, null); break;
                case Difficulty.Average: bestCol = AlphaBetaAlgo.AlphaBeta(game, 5, null); break;
                case Difficulty.Hard: bestCol = AlphaBetaAlgo.AlphaBeta(game, 8, null); break;
            }
            return bestCol;
        }
    }
}