using System;
using System.Collections.Generic;

/*************************************************************************************
 *
 *  P4Lib
 *  Author : Bidou (http://www.csharpfr.com/auteurdetail.aspx?ID=13319)
 *  Blog   : http://blogs.developpeur.org/bidou/
 *  Date   : February 2007
 * 
 *************************************************************************************/

namespace P4Lib
{
    /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /// <summary>
    /// Implements the Alpha-Beta algorithm.
    /// </summary>
    /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    internal class AlphaBetaAlgo
    {
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Create a new AlphaBetaAlgo object.
        /// </summary>
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        public AlphaBetaAlgo()
        {
        }

        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Return the best column.
        /// </summary>
        /// <param name="game"> The current game. </param>
        /// <param name="maxDepth"> The maximum depth (in tree). </param>
        /// <param name="notUsed"> Pass null please. </param>
        /// <returns> The best column. </returns>
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        public static int AlphaBeta(Game game, byte maxDepth, object notUsed)
        {
            AlphaBetaAlgo ab = new AlphaBetaAlgo();
            return ab.AlphaBeta(game, maxDepth);
        }

        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Return the best column.
        /// </summary>
        /// <param name="game"> The current game. </param>
        /// <param name="maxDepth"> The maximum depth (in tree). </param>
        /// <returns> The best column. </returns>
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        public int AlphaBeta(Game game, byte maxDepth)
        {
            List<Game> gResult = new List<Game>(); // The best points
            int max = Int32.MinValue;
            foreach (Game curGame in game.GetSuccessors(PlayerType.Computer))
            {
                int sonValue = AlphaBetaInternal(curGame, Int32.MinValue, Int32.MaxValue, maxDepth);
                if (sonValue > max) // We have a new best point
                {
                    gResult.Clear();
                    max = sonValue;
                }
                if (sonValue == max) gResult.Add(curGame);
            }
            return gResult[Tools.ChooseColumn(gResult.Count)].BestColumn; // Choose one between the bests
        }

        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        /// <summary>
        /// Internal recursive function (iterate in the tree).
        /// </summary>
        /// <param name="game"> The current game. </param>
        /// <param name="alpha"> The alpha value. </param>
        /// <param name="beta"> The beta value. </param>
        /// <param name="maxDepth"> The maximum depth in the tree. </param>
        /// <returns> The best value. </returns
        /// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        private int AlphaBetaInternal(Game game, int alpha, int beta, int maxDepth)
        {
            if (game.IsLeaf || maxDepth <= 0) return game.Value;
            else
            {
                if (game.CurrentPlayer == PlayerType.Human)
                {
                    foreach (Game curGame in game.GetSuccessors(PlayerType.Human))
                    {
                        int move = AlphaBetaInternal(curGame, alpha, beta, maxDepth - 1);
                        beta = Math.Min(beta, move);
                        if (alpha >= beta) return beta;
                    }
                    return beta;
                }
                else
                {
                    foreach (Game curGame in game.GetSuccessors(PlayerType.Computer))
                    {
                        int move = AlphaBetaInternal(curGame, alpha, beta, maxDepth - 1);
                        alpha = Math.Max(alpha, move);
                        if (alpha >= beta) return alpha;
                    }
                    return alpha;
                }
            }
        }
    }
}