using System;
using System.Drawing;
using System.Windows.Forms;
using P4Lib;
using P4Ctrl;

// No comment for the window application...
namespace P4Win
{
    public partial class frmMain : Form
    {
        public static UserOptions options = new UserOptions();

        public frmMain()
        {
            this.InitializeComponent();
            this.lblInfos.Text = options.ToString();
        }

        #region BTNCLICK

        private void btnRestart_Click(object sender, EventArgs e)
        {
            this.toolStripStatusLblInfos.ForeColor = Color.Black;
            if (this.btnStart.Text == "Stop")
            {
                this.btnStart.Text = "Start";
                this.p4Grid.Stop();
            }
            else
            {
                this.btnStart.Text = "Stop";
                this.toolStripStatusLblInfos.Text = "Ready";
                this.p4Grid.HumanColor = options.playerColor;
                this.p4Grid.Restart(options.computerPlaysFirst, options.diff);
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion

        #region Grid Handler

        private void p4Grid_UserWin(PlayerType playerType)
        {
            // Anonymous invocation
            this.btnStart.Invoke(new EventHandler(delegate(object o, EventArgs e) { this.btnStart.Text = "Start"; }));
            // 'Normal' invokation
            this.statusStrip.Invoke(new EventHandler(this.WinnerStrip), playerType);
        }

        private void p4Grid_ComputerStartsThinking(object sender, EventArgs e)
        {
            this.toolStripStatusLblInfos.Text = "Computer is playing...";
            this.toolStripPrgBarWait.Visible = true;
            this.toolStripPrgBarWait.MarqueeAnimationSpeed = 50;
        }

        private void p4Grid_ComputerEndsThinking(object sender, EventArgs e)
        {
            this.toolStripPrgBarWait.ProgressBar.Invoke(new EventHandler(this.StopAnimation));
        }

        #endregion

        #region Invokation

        private void StopAnimation(object o, EventArgs e)
        {
            this.toolStripStatusLblInfos.Text = "It's your turn: waiting...";
            this.toolStripPrgBarWait.Visible = false;
            this.toolStripPrgBarWait.MarqueeAnimationSpeed = 0;
        }

        private void WinnerStrip(object o, EventArgs e)
        {
            PlayerType playerType = (PlayerType)o;
            this.toolStripStatusLblInfos.Text = playerType != PlayerType.NoBody ? string.Format("{0} wins !!", playerType) : "Deuce";
            this.toolStripStatusLblInfos.ForeColor = Color.Red;
        }

        #endregion

        #region Menu

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void mnuFileOptions_Click(object sender, EventArgs e)
        {
            if (this.p4Grid.IsFinished)
            {
                using (frmOptions frmOp = new frmOptions())
                {
                    if (frmOp.ShowDialog() == DialogResult.OK) this.lblInfos.Text = options.ToString();
                }
            }
            else MessageBox.Show("Pas disponible durant une partie!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        private void mnuAboutAbout_Click(object sender, EventArgs e)
        {
            string txt = String.Format("                                        - By Bidou -\n\n" +
                                        "Date : February 2007\n" +
                                        "CS    : http://www.csharpfr.com/auteurdetail.aspx?ID=13319");
            MessageBox.Show(txt, "About");
        }

        #endregion

    }
}